@@ -1,12 +1,13 @@
-/*$Id: Firekeeper.cpp,v 1.19 2006/08/20 16:39:54 jwrobel Exp $*/
+/*$Id: Firekeeper.cpp,v 1.24 2006/12/21 21:03:26 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
  *  Copyright (C) 2006 Jan Wrobel <wrobel@blues.ath.cx>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License Version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,47 +36,58 @@
 #include "Error.h"
 #include "Common.h"
 
+
 static const nsIID kIComponentRegistrarIID = NS_ICOMPONENTREGISTRAR_IID;
 static const nsCID kfkHttpHandlerCID  = FK_HTTPHANDLER_CID;
 static const nsCID kfkHttpsHandlerCID = FK_HTTPSHANDLER_CID;
 
+/*Loaded rules*/
 Rules *rules = nsnull;
+
 nsCOMPtr<fkIJudge> fkJudge = nsnull;
-Firekeeper *firekeeper = nsnull;
 
 /*TODO: this should be configurable*/
 const char *rulesOrder[]={"pass", "drop", "alert"};
 
-Firekeeper::Firekeeper(): httpHandler(nsnull), httpsHandler(nsnull)
-{	
+Firekeeper::Firekeeper()
+{
+	this->httpHandler = nsnull;
+	this->httpsHandler = nsnull;
 	TRACE("constructor %08x", this);
 }
 
 Firekeeper::~Firekeeper()
 {
 	TRACE("destructor %08x", this);
-	firekeeper = nsnull;
+	FK_ASSERT(false);
+	//firekeeper = nsnull;
 }
 
 NS_IMPL_ISUPPORTS2(Firekeeper, fkIFirekeeper,
 		   nsIObserver)
-
-
+	
+/*
+  Return global Firekeeper object (singleton)
+ */
 Firekeeper*
 Firekeeper::GetInstance()
 {
+	static Firekeeper *firekeeper;
+	
 	TRACE("");
 	if (firekeeper == nsnull){
 		firekeeper = new Firekeeper();
 		if (!firekeeper)
 			return nsnull;
-
+		
 		/*don't destroy firekeeper*/
 		NS_ADDREF(firekeeper);	
 			
 		nsresult rv = firekeeper->init();
-		if (NS_FAILED(rv))
+		if (NS_FAILED(rv)){
+			//TODO: delete firekeeper; ...
 			return nsnull;
+		}
 	}
 	
 	NS_IF_ADDREF(firekeeper);
@@ -83,6 +95,10 @@ Firekeeper::GetInstance()
 }
  
 
+/*
+  Init Firekeeper. Replace oryginal Mozilla's HTTP and HTTPS handlers
+  with new ones, able to trace and check traffic.
+ */
 NS_IMETHODIMP 
 Firekeeper::init()
 {
@@ -90,13 +106,12 @@ Firekeeper::init()
 	rules = nsnull;	
 	rules = new Rules();
 	if (!rules || rules->init() < 0)
-		return NS_ERROR_OUT_OF_MEMORY;		
+		return NS_ERROR_OUT_OF_MEMORY;
 	
-
 	nsCOMPtr<nsIServiceManager> servman; 
 	nsresult rv = NS_GetServiceManager(getter_AddRefs(servman)); 
 	if (NS_FAILED(rv)){
-		TRACE("Can't get ServiceManager");
+		TRACE("Can't get ServiceManager %08x", rv);
 		return rv;
 	}
 
@@ -106,19 +121,22 @@ Firekeeper::init()
 		TRACE("Can't get ComponentManager");
 		return rv;
 	}
-	
+
 	nsCOMPtr<nsIComponentRegistrar> compreg;	
-	rv = compmgr->QueryInterface(kIComponentRegistrarIID, getter_AddRefs(compreg));
+	rv = compmgr->QueryInterface(kIComponentRegistrarIID, 
+				     getter_AddRefs(compreg));
 	if (NS_FAILED(rv)){
 		TRACE("Can't get ComponentRegistrar");
 		return rv;
 	}
-
+	
 	/*register new HTTP handler*/
 	nsCOMPtr<nsIHttpProtocolHandler> oldHandler;
-	rv = servman->GetServiceByContractID(HTTPHANDLER_CONTRACTID,
-					     NS_GET_IID(nsIHttpProtocolHandler),
-					     getter_AddRefs(oldHandler));
+	rv = servman->
+		GetServiceByContractID(HTTPHANDLER_CONTRACTID,
+				       NS_GET_IID(nsIHttpProtocolHandler),
+				       getter_AddRefs(oldHandler));
+	
 	if (NS_FAILED(rv))
 		return rv;
 	
@@ -126,54 +144,69 @@ Firekeeper::init()
 		TRACE("oldHandler == NULL");
 		return NS_ERROR_FAILURE;
 	}
+	TRACE("got old handler %08x", &*oldHandler);
+	
 	httpHandler = new fkHttpHandler();
 	if (!httpHandler){
 		TRACE("Can't create httpHandler");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
+	
 	rv = httpHandler->init(oldHandler);
 	if (NS_FAILED(rv)){
 		TRACE("Can't init http handler");
 		return rv;
 	}	
+	
 	NS_ADDREF(httpHandler);
 	
-	fkHttpHandlerFactory *httpHandlerFactory = new fkHttpHandlerFactory(httpHandler);
+	fkHttpHandlerFactory *httpHandlerFactory = 
+		new fkHttpHandlerFactory(httpHandler);
+	
 	if (!httpHandlerFactory){
 		TRACE("Can't create httpHandlerFactory");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
-	NS_ADDREF(httpHandlerFactory);
+	NS_ADDREF(httpHandlerFactory);	
 	
+	
+	
+	TRACE("registering %s", HTTPHANDLER_CONTRACTID);
 	rv = compreg->RegisterFactory(kfkHttpHandlerCID,
 				      "fkHttpHandler",
 				      HTTPHANDLER_CONTRACTID,
-				      httpHandlerFactory);
+				      httpHandlerFactory);	
 	if (NS_FAILED(rv)){
 		TRACE("Can't register factory");
 		return rv;
 	}
 
-	TRACE("http handler replaced");
+	TRACE("http handler replaced getting %s", HTTPSHANDLER_CONTRACTID);
 
 	
 	/*register new HTTPS handler*/
 	nsCOMPtr<nsIHttpProtocolHandler> oldHttpsHandler;
-	rv = servman->GetServiceByContractID(HTTPSHANDLER_CONTRACTID,
-					     NS_GET_IID(nsIHttpProtocolHandler),
-					     getter_AddRefs(oldHttpsHandler));
-	if (NS_FAILED(rv))
+	rv = servman->
+		GetServiceByContractID(HTTPSHANDLER_CONTRACTID, 
+				       NS_GET_IID(nsIHttpProtocolHandler),
+				       getter_AddRefs(oldHttpsHandler));
+	
+	TRACE("got it %08x", &*oldHttpsHandler);
+	if (NS_FAILED(rv)){
 		return rv;
+	}
 	
 	if (!oldHttpsHandler){
 		TRACE("oldHttpsHandler == NULL");
 		return NS_ERROR_FAILURE;
 	}	
+	TRACE("creating new fkHttpsHandler");
 	httpsHandler = new fkHttpHandler();
 	if (!httpsHandler){
 		TRACE("Can't create httpsHandler");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
+	TRACE("initializing HttpsHandler");
 	rv = httpsHandler->init(oldHttpsHandler);
 	if (NS_FAILED(rv)){
 		TRACE("Can't init https handler");
@@ -182,7 +215,8 @@ Firekeeper::init()
 	
 	NS_ADDREF(httpsHandler);
 	
-	fkHttpHandlerFactory *httpsHandlerFactory = new fkHttpHandlerFactory(httpsHandler);
+	fkHttpHandlerFactory *httpsHandlerFactory =
+		new fkHttpHandlerFactory(httpsHandler);
 	if (!httpsHandlerFactory){
 		TRACE("Can't create httpsHandlerFactory");
 		return NS_ERROR_OUT_OF_MEMORY;
@@ -202,16 +236,23 @@ Firekeeper::init()
 	return NS_OK;
 }
 
+//-----------------------------------------------------------------------------
+// Firekeeper::fkIFirekeeper
+//-----------------------------------------------------------------------------
+
+
 NS_IMETHODIMP 
 Firekeeper::AddRule(const char *rule, char **errmsg)
 {	
 	char *error = nsnull;
+	TRACE("");
 	if (!errmsg)
 		return NS_ERROR_NULL_POINTER;
 	
 	/*TODO: don't create parser here*/
-	RulesParser p(&error);
-	if (error){
+	RulesParser p;
+	
+	if (p.init(&error) < 0){
 		err_msg(error);
 		return NS_ERROR_FAILURE;
 	}
@@ -223,7 +264,7 @@ Firekeeper::AddRule(const char *rule, char **errmsg)
 		if (!*errmsg)
 			return NS_ERROR_OUT_OF_MEMORY;
 
-		/*can't return NS_ERROR_FAILURE here it would
+		/*Can't return NS_ERROR_FAILURE here, it would
 		  make errmsg variable inaccessible in javascript*/
 		return NS_OK;
 	}
@@ -233,16 +274,16 @@ Firekeeper::AddRule(const char *rule, char **errmsg)
 	return NS_OK;
 }
 
-/* void addRules (in string rules, out string errmsg); */
 NS_IMETHODIMP 
 Firekeeper::AddRules(const char *rulesbuf, char **errmsg)
 {
 	char *error;
+	TRACE("");
 	if (!errmsg)
 		return NS_ERROR_NULL_POINTER;
 	
-	RulesParser p(&error);
-	if (error){
+	RulesParser p;
+	if (p.init(&error)){
 		err_msg(error);
 		return NS_ERROR_FAILURE;
 	}
@@ -260,49 +301,61 @@ Firekeeper::AddRules(const char *rulesbuf, char **errmsg)
 	return NS_OK;
 }
 
-
-/* void setJudge (in fkIJudge judge); */
 NS_IMETHODIMP 
 Firekeeper::SetJudge(fkIJudge *judge)
 {
+	TRACE("");
 	fkJudge = judge;
 	return NS_OK;
 }
 
-
-NS_IMETHODIMP 
-Firekeeper::Observe(nsISupports *subject, const char* topic, const PRUnichar* data) 
-{
-	if (!topic || strcmp(topic, "xpcom-startup") != 0)
-		return NS_OK;	
-	return NS_OK;
-}
-
-/* void enable (in boolean tracing); */
 NS_IMETHODIMP 
 Firekeeper::Enable(PRBool tracingEnabled)
 {
+	TRACE("");
+	if (!httpHandler){
+		TRACE("null httpHandler");
+		return NS_ERROR_FAILURE;
+	}
 	httpHandler->enableTracing(tracingEnabled);
 	return NS_OK;
 }
 
-/* void isEnabled (out boolean tracing); */
 NS_IMETHODIMP 
 Firekeeper::IsEnabled(PRBool *tracingEnabled)
 {
+	TRACE("");
 	if (!tracingEnabled)
 		return NS_ERROR_NULL_POINTER;
+	if (!httpHandler)
+		return NS_ERROR_FAILURE;
 	*tracingEnabled = httpHandler->isTracingEnabled();
 	return NS_OK;
 }
 
-/* void resetRules (); */
 NS_IMETHODIMP 
 Firekeeper::ResetRules()
 {
+	TRACE("");
 	if (rules == nsnull)
 		return NS_ERROR_FAILURE;
 	
 	rules->resetRules();
+	TRACE("ok");
+	return NS_OK;
+}
+
+
+//-----------------------------------------------------------------------------
+// Firekeeper::nsIObserver
+//-----------------------------------------------------------------------------
+NS_IMETHODIMP 
+Firekeeper::Observe(nsISupports *subject, const char* topic,
+		    const PRUnichar* data) 
+{
+	TRACE("");
+	if (!topic || strncmp(topic, "xpcom-startup", 13) != 0){
+		return NS_OK;	
+	}
 	return NS_OK;
 }
