@@ -1,4 +1,4 @@
-/*$Id: RulesParser.cpp,v 1.15 2006/09/03 19:00:50 jwrobel Exp $*/
+/*$Id: RulesParser.cpp,v 1.21 2006/12/21 08:40:15 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -9,8 +9,9 @@
  *  Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License Version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +23,15 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * Routines to parse files with rules. Every rule should occupy single line.
+ * Empty lines and lines starting with '#' character are permitted.
+ * When some rules are incorrect parser tries to read other rules anyway.
+ *
+ */
+
 #include "Common.h"
 #include "RulesParser.h"
-#include "RuleSyntax.h"
 #include "Wrap.h"
 #include "Error.h"
 
@@ -32,7 +39,7 @@
 
 #ifndef HAVE_VSNPRINTF
 
-#ifdef HAVE__VSNPRINTF /*MSVC defines this*/
+#ifdef HAVE__VSNPRINTF /*MSVC uses this*/
 int (*vsnprintf)(char *, size_t, const char *, va_list) = _vsnprintf;
 int (*snprintf)(char *, size_t, const char *, ...) = _snprintf;
 #else /*HAVE__VSNPRINTF*/
@@ -43,21 +50,78 @@ int (*snprintf)(char *, size_t, const char *, ...) = _snprintf;
 #endif /*HAVE_VSNPRINTF*/
 
 
+/**** Patterns to parse rules ****/
 /*
- *@error - set to not null if there are some problems compiling 
- *         regular expressions that describe rule syntax. This
- *         generally shouldn't occur, but maybe is possible if
- *         there are problems with pcre library.
- */
-RulesParser::RulesParser(char **error)
+  Firekeeper uses pcre library to parse rules.
+  
+  
+  alert (msg: " "; url_content: ; headers_content: ; body_conent: ; url_re:; headers_re: ; body_re: ; reference:cve,2004-0258; reference:bugtraq,9579; fid:2439; rev:5;)
+  pass   
+  drop
+*/
+
+
+/*Check if rule has form 'action (options);' 
+  end extract action and options*/
+const char *rulePattern = "\
+(?x)         #turn on extended syntax \n\
+^            #whole line must match \n\
+[[:space:]]* #white spaces at the beginning of a rule are ok\n\
+(            \
+[[:word:]]+  #action\n\
+)            \
+[[:space:]]* \
+\\(          \
+(            \
+.*           #rule options \n\
+)            \
+\\)          \
+[[:space:]]* #white spaces at the end of a rule are ok\n\
+$            \
+";
+
+/*extract option value from options string without quotations*/
+#define OPTION_VALUE         "[[:space:]]*\"(.*?[^\\\\])\"[[:space:]]*;"
+/*extract option value from options string with quotations*/
+#define OPTION_VALUE_WITH_QT "[[:space:]]*(\".*?[^\\\\]\")[[:space:]]*;"
+
+const char *msgPattern = "msg:" OPTION_VALUE;
+
+const char *referencePattern = "\
+(?x)                             \
+reference:                 #pointer to site that has more info about alert\n\
+[[:space:]]*                     \
+(bugtraq|cve|nessus|mcafee|url)  #same systems that Snort supports.\n\
+[[:space:]]*,[[:space:]]*        \
+(.*?)                            \
+[[:space:]]*;                    \
+";
+
+const char *fidPattern = "fid:[[:space:]]*([[:digit:]]+)[[:space:]]*;";
+const char *revPattern = "rev:[[:space:]]*([[:digit:]]+)[[:space:]]*;";
+
+/*nocase options is allowed after *_content options */
+#define OPTIONAL_NOCASE "([[:space:]]*nocase[[:space:]]*;)?"
+
+const char *urlContentPattern = 
+"url_content:" OPTION_VALUE_WITH_QT OPTIONAL_NOCASE;
+
+const char *headersContentPattern = 
+"headers_content:" OPTION_VALUE_WITH_QT OPTIONAL_NOCASE;
+
+const char *bodyContentPattern = 
+"body_content:" OPTION_VALUE_WITH_QT OPTIONAL_NOCASE;
+
+const char *headersRegPattern = "headers_re:" OPTION_VALUE;
+
+const char *bodyRegPattern = "body_re:" OPTION_VALUE;
+
+const char *urlRegPattern = "url_re:" OPTION_VALUE;
+
+/*********************************/
+
+RulesParser::RulesParser()
 {
-	memset(this, 0, sizeof(*this));
-	*error = NULL;
-	compileREs();
-	if (strlen(errmsg)){
-		*error = errmsg;
-		freeREs();
-	}
 };
 
 RulesParser::~RulesParser()
@@ -65,12 +129,36 @@ RulesParser::~RulesParser()
 	freeREs();
 };
 
+
+/*
+ *@error: set to not null if there are some initialization 
+ *        problems
+ */
+int RulesParser::init(char **error)
+{
+	if (!error)
+		return -1;
+	
+	memset(this, 0, sizeof(*this));
+	*error = NULL;
+	compileREs();
+	
+	if (strlen(errmsg)){
+		*error = errmsg;
+		freeREs();
+		return -1;
+	}
+	return 0;
+};
+
+/*Append error message to error buffer*/
 void RulesParser::err_append(const char *fmt, ...)
 {
 	char buf[ERRBUFLEN];
 	va_list ap;
 	va_start(ap, fmt);
 	vsnprintf(buf, ERRBUFLEN, fmt, ap);
+	buf[ERRBUFLEN - 1] = 0;
 	va_end(ap);
 	int l;
 	
@@ -81,7 +169,8 @@ void RulesParser::err_append(const char *fmt, ...)
 	errmsg[ERRBUFLEN - 1] = 0;
 }
 
-	
+
+/*filter empty lines*/	
 bool RulesParser::emptyString(char *p)
 {
 	for(; *p; p++)
@@ -90,8 +179,8 @@ bool RulesParser::emptyString(char *p)
 	return true;
 }
 
-
-bool RulesParser::comentedString(char *p)
+/*filter commented and empty lines*/
+bool RulesParser::commentedOrEmptyString(char *p)
 {
 	for(; *p; p++)
 		if (!isspace(*p)){
@@ -107,15 +196,16 @@ int RulesParser::compileRE(pcre **re, const char *pattern, const char *name)
 	const char *error;
 	int erroffset;
 
-	*re = pcre_compile(pattern,              /* the pattern */
-			   0,                    /* default options */
-			   &error,               /* for error message */
-			   &erroffset,           /* for error offset */
-			   NULL);                /* use default character tables */
+	*re = pcre_compile(pattern,         /* the pattern */
+			   0,               /* default options */
+			   &error,          /* for error message */
+			   &erroffset,      /* for error offset */
+			   NULL);           /* use default character tables */
 	
 	if (error){
-		err_append("Fatal error: '%s' invalid %s rule syntax at position %d: %s",
-			   pattern, name, erroffset, error);
+		err_append("Fatal error: '%s' invalid %s rule syntax "
+			   "at position %d: %s", pattern, name, 
+			   erroffset, error);
 		return -1;
 	}
 
@@ -123,25 +213,6 @@ int RulesParser::compileRE(pcre **re, const char *pattern, const char *name)
 	return 0;
 }
 
-int RulesParser::compileREs()
-{
-	
-	if (compileRE(&ruleRE, rulePattern, "general") ||
-	    compileRE(&msgRE, msgPattern, "msg") ||
-	    compileRE(&referenceRE, referencePattern, "reference") ||
-	    compileRE(&fidRE, fidPattern, "fid") ||
-	    compileRE(&revRE, revPattern, "rev") ||
-	    compileRE(&headerRE, headersRegPattern, "headers_re") ||
-	    compileRE(&bodyRE, bodyRegPattern, "body_re") ||
-	    compileRE(&urlRE, urlRegPattern, "url_re") ||
-	    compileRE(&urlContentRE, urlContentPattern, "url_content") ||
-	    compileRE(&headersContentRE, headersContentPattern, "headers_content") ||
-	    compileRE(&bodyContentRE, bodyContentPattern, "body_content")
-	    )
-		return -1;	
-	return 0;
-}
-
 void RulesParser::freeRE(pcre **re)
 {
 	if (*re){
@@ -151,7 +222,47 @@ void RulesParser::freeRE(pcre **re)
 }
 
 
-/*lots of Snort code*/
+/*Compile regular expressions used to parse rules and separate rules options*/
+int RulesParser::compileREs()
+{
+	
+	if (compileRE(&ruleRE, rulePattern, "general") ||
+	    compileRE(&msgRE, msgPattern, "msg") ||
+	    compileRE(&referenceRE, referencePattern, "reference") ||
+	    compileRE(&fidRE, fidPattern, "fid") ||
+	    compileRE(&revRE, revPattern, "rev") ||
+	    compileRE(&headerRE, headersRegPattern, "headers_re") ||
+	    compileRE(&bodyRE, bodyRegPattern, "body_re") ||
+	    compileRE(&urlRE, urlRegPattern, "url_re") ||
+	    compileRE(&urlContentRE, urlContentPattern, "url_content") ||
+	    compileRE(&headersContentRE, headersContentPattern, 
+		      "headers_content") ||
+	    compileRE(&bodyContentRE, bodyContentPattern, "body_content")
+	    /*Remember to free allocated memory in freeREs*/
+	    )
+		return -1;	
+	return 0;
+}
+
+void RulesParser::freeREs()
+{
+	freeRE(&ruleRE);
+	freeRE(&msgRE);
+	freeRE(&referenceRE);
+	freeRE(&fidRE);
+	freeRE(&revRE);
+	freeRE(&headerRE);
+	freeRE(&bodyRE);
+	freeRE(&urlRE);
+	freeRE(&headersContentRE);
+	freeRE(&bodyContentRE);
+	freeRE(&urlContentRE);
+}
+
+/*Lots of Snort code.
+  Parse regular expression specified with 'url_re:', 'headers_re:' 
+  or 'body_re:' option and compile it.
+*/
 bool RulesParser::parseRE(char *data, const char *name, 
 			  RE *result, char *errbuf, int errbuflen)
 {
@@ -162,16 +273,14 @@ bool RulesParser::parseRE(char *data, const char *name,
     int erroffset;
     int compile_flags = 0;
     
-    if(data == NULL) 
-    {
+    if(data == NULL){
 	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
 		     "re requires a regular expression",
 		     name, data);
 	    goto error;
     }
 
-    if(!(free_me = strdup(data)))
-    {
+    if(!(free_me = strdup(data))){
 	    snprintf(errbuf, errbuflen, "Out of memory");
 	    goto error;
     }
@@ -185,11 +294,10 @@ bool RulesParser::parseRE(char *data, const char *name,
     
     /* 'm//' or just '//' */
         
-    if(*re == 'm')
-    {
+    if(*re == 'm'){
 	    re++;
 	    if(! *re) goto syntax;
-        
+	    
 	    /* Space as a ending delimiter?  Uh, no. */
 	    if(isspace((int)*re)) goto syntax;
 	    /* using R would be bad, as it triggers RE */
@@ -237,11 +345,12 @@ bool RulesParser::parseRE(char *data, const char *name,
     /* now compile the re */
     TRACE("pcre: compiling %s", re);
     
-    result->re_compiled = pcre_compile(re, compile_flags, &error, &erroffset, NULL);
+    result->re_compiled = pcre_compile(re, compile_flags, &error, 
+				       &erroffset, NULL);
 
     if(result->re_compiled == NULL){
-	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s' at position %d: %s",
-		     name, data, erroffset, error);
+	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s' "
+		     "at position %d: %s", name, data, erroffset, error);
 	    goto error;
     }
 
@@ -255,8 +364,8 @@ bool RulesParser::parseRE(char *data, const char *name,
     return true;
 
  syntax:
-    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': unable to parse regular expression",
-	     name, data);
+    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
+	     "unable to parse regular expression", name, data);
  error:
     if(free_me) 
 	    free(free_me);
@@ -264,14 +373,18 @@ bool RulesParser::parseRE(char *data, const char *name,
 }
 
 
-/****************************************************************************
+/*
+ * Parse and validate content specified with 'url_content:', 'headers_content:'
+ * or 'body_content:' option.
  *
  * Slightly modified ParsePattern(char * ...) function from Snort 
  * ./detection-plugins/sp_pattern_match.c 
- * Author thinks this code is ugly, it works - good enough for me
- ***************************************************************************/
+ *TODO: Split this code. Snort uses smaller indentations so its version
+ *isn't so unreadable.
+ */
 bool RulesParser::parseContent(char *rule, const char *name,
-			       PatternMatchData *ds_idx, char *errbuf, int errbuflen)
+			       PatternMatchData *ds_idx, 
+			       char *errbuf, int errbuflen)
 {
     /*TODO: it was unsigned in original code*/
     char tmp_buf[2048];
@@ -297,15 +410,15 @@ bool RulesParser::parseContent(char *rule, const char *name,
     memset(tmp_buf, 0, 2048);
     
     if(rule == NULL){	    
-	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s' empty content",
-		     name, rule);
+	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s' "
+		     "empty content", name, rule);
 	    return false;
     }
 
     while(isspace((int)*rule))
         rule++;
 
-    /*not supported
+    /*not supported yet
     if(*rule == '!')
     {
         exception_flag = 1;
@@ -315,10 +428,9 @@ bool RulesParser::parseContent(char *rule, const char *name,
     /* find the start of the data */
     start_ptr = strchr(rule, '"');
 
-    if(start_ptr == NULL)
-    {
-	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-		     "content data needs to be enclosed in quotation marks (\")",
+    if(start_ptr == NULL){
+	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': content "
+		     "data needs to be enclosed in quotation marks (\")",
 		     name, rule);
 	    return false;
     }
@@ -329,10 +441,9 @@ bool RulesParser::parseContent(char *rule, const char *name,
     /* find the end of the data */
     end_ptr = strrchr(start_ptr, '"');
 
-    if(end_ptr == NULL)
-    {
-	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-		     "content data needs to be enclosed in quotation marks (\")",
+    if(end_ptr == NULL){
+	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': content "
+		     "data needs to be enclosed in quotation marks (\")",
 		     name, rule);
 	    return false;
     }
@@ -344,10 +455,9 @@ bool RulesParser::parseContent(char *rule, const char *name,
      * double quote? */
     tmp = end_ptr + 1;
     while (*tmp != '\0' && isspace ((int)*tmp))
-        tmp++;
+	    tmp++;
 
-    if (strlen (tmp) > 0)
-    {
+    if (strlen (tmp) > 0){
 	    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
 		     "missing semicolon after trailing double quote.",
 		     name, rule);
@@ -372,188 +482,160 @@ bool RulesParser::parseContent(char *rule, const char *name,
     memset(hex_buf, '0', 2);
 
     /* BEGIN BAD JUJU..... */
-    while(idx < end_ptr)
-    {
-        TRACE("processing char: %c", *idx);
-        switch(*idx)
-        {
+    while(idx < end_ptr){
+	    TRACE("processing char: %c", *idx);
+	    switch(*idx){
             case '|':
-                TRACE("Got bar... ");
-                if(!literal)
-                {
-                    TRACE("not in literal mode... ");
-                    if(!hexmode)
-                    {
-                        TRACE("Entering hexmode");
-                        hexmode = 1;
-                    }
-                    else
-                    {
-                        TRACE("Exiting hexmode");
+		    TRACE("Got bar... ");
+		    if(!literal){
+			    TRACE("not in literal mode... ");
+			    if(!hexmode){
+				    TRACE("Entering hexmode");
+				    hexmode = 1;
+			    }
+			    else{
+				    TRACE("Exiting hexmode");
 
-                        /*
-                        **  Hexmode is not even.
-                        */
-                        if(!hexsize || hexsize % 2)
-                        {
-				snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-					 "content hexmode argument has invalid "
-					 "number of hex digits.  The argument must "
-					 "contain a full even byte string.",
-					 name, rule);
-				return false;
-                        }
+				    /*
+				    **  Hexmode is not even.
+				    */
+				    if(!hexsize || hexsize % 2){
+					    snprintf(errbuf, errbuflen, 
+						     "Incorrect '%s' \
+syntax '%s': content hexmode argument has invalid number of hex digits. \
+The argument must contain a full even byte string.", name, rule);
+					    return false;
+				    }
 
-                        hexmode = 0;
-                        pending = 0;
-                    }
+				    hexmode = 0;
+				    pending = 0;
+			    }
 
-                    if(hexmode)
-                        hexsize = 0;
-                }
-                else
-                {
-                    TRACE("literal set, Clearing");
-                    literal = 0;
-                    tmp_buf[dummy_size] = start_ptr[cnt];
-                    dummy_size++;
-                }
+			    if(hexmode)
+				    hexsize = 0;
+		    }
+		    else{
+			    TRACE("literal set, Clearing");
+			    literal = 0;
+			    tmp_buf[dummy_size] = start_ptr[cnt];
+			    dummy_size++;
+		    }
 
-                break;
+		    break;
 
             case '\\':
 		    TRACE("Got literal char... ");
 
-                if(!literal)
-                {
+		    if(!literal){
                     /* Make sure the next char makes this a valid
                      * escape sequence.
                      */
-                    if (idx [1] != '\0' && strchr ("\\\":;", idx [1]) == NULL)
-                    {
-			    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-				     "bad escape sequence starting with '%s'",
-				     name, rule, idx);
-			    return false;
-                    }
-
-                    TRACE("Setting literal");
-
-                    literal = 1;
-                }
-                else
-                {
-                    TRACE("Clearing literal");
-                    tmp_buf[dummy_size] = start_ptr[cnt];
-                    literal = 0;
-                    dummy_size++;
-                }
-
-                break;
-            case '"':
-                if (!literal) {
-			snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-				 "Non-escaped '\"' character",
-				 name, rule);
-			return false;
-                }
-                /* otherwise process the character as default */
-            default:
-                if(hexmode)
-                {
-                    if(isxdigit((int) *idx))
-                    {
-                        hexsize++;
-
-                        if(!pending)
-                        {
-                            hex_buf[0] = *idx;
-                            pending++;
-                        }
-                        else
-                        {
-                            hex_buf[1] = *idx;
-                            pending--;
-
-                            if(dummy_idx < dummy_end)
-                            {                            
-			            tmp_buf[dummy_size] = (unsigned char) 
-                                    strtol(hex_buf, (char **) NULL, 16)&0xFF;
-
-                                dummy_size++;
-                                memset(hex_buf, 0, 3);
-                                memset(hex_buf, '0', 2);
-                            }
-                            else
-                            {
-				    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax: "
-					     "buffer overflow, make a smaller "
-					     "pattern please! (Max size = 2048)",
-					     name);
+			    if (idx [1] != '\0' && 
+				strchr ("\\\":;", idx [1]) == NULL){
+				    snprintf(errbuf, errbuflen, "Incorrect \
+'%s' syntax '%s': bad escape sequence starting with '%s'", name, rule, idx);
 				    return false;
-                            }
-                        }
-                    }
-                    else
-                    {
-                        if(*idx != ' ')
-                        {
-				snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-					     "What is this "
-					 "\"%c\"(0x%X) doing in your binary "
-					 "buffer?  Valid hex values only please"
-					 "(0x0 - 0xF) Position: %d",
-					 name, rule, (char) *idx, (char) *idx, cnt);
-				return false;
-                        }
-                    }
-                }
-                else
-                {
-                    if(*idx >= 0x1F && *idx <= 0x7e)
-                    {
-                        if(dummy_idx < dummy_end)
-                        {
-                            tmp_buf[dummy_size] = start_ptr[cnt];
-                            dummy_size++;
-                        }
-                        else
-                        {
-				snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-					 "dummy buffer overflow", name, rule);
-				return false;
-                        }
+			    }
 
-                        if(literal)
-                        {
-                            literal = 0;
-                        }
-                    }
-                    else
-                    {
-                        if(literal)
-                        {
-                            tmp_buf[dummy_size] = start_ptr[cnt];
-                            dummy_size++;
-                            TRACE("Clearing literal");
-                            literal = 0;
-                        }
-                        else
-                        {
-				snprintf(errbuf, errbuflen, "Incorrect '%s' syntax '%s': "
-					 "character value out of range, try a binary buffer",
-					 name, rule);
-				return false;
-                        }
-                    }
-                }
+			    TRACE("Setting literal");
+			    
+			    literal = 1;
+		    }
+		    else{
+			    TRACE("Clearing literal");
+			    tmp_buf[dummy_size] = start_ptr[cnt];
+			    literal = 0;
+			    dummy_size++;
+		    }
 
-                break;
-        }
+		    break;
+            case '"':
+		    if (!literal) {
+			    snprintf(errbuf, errbuflen, "Incorrect '%s' syntax\
+  '%s': Non-escaped '\"' character", name, rule);
+			    return false;
+		    }
+		    /* otherwise process the character as default */
+            default:
+		    if(hexmode){
+			    if(isxdigit((int) *idx)){
+				    hexsize++;
 
-        dummy_idx++;
-        idx++;
-        cnt++;
+				    if(!pending){
+					    hex_buf[0] = *idx;
+					    pending++;
+				    }
+				    else{
+					    hex_buf[1] = *idx;
+					    pending--;
+					    
+					    if(dummy_idx < dummy_end){
+						    tmp_buf[dummy_size] = (unsigned char) strtol(hex_buf, (char **) NULL, 16)&0xFF;
+
+						    dummy_size++;
+						    memset(hex_buf, 0, 3);
+						    memset(hex_buf, '0', 2);
+					    }
+					    else{
+						    snprintf(errbuf, errbuflen,
+							     "Incorrect '%s' \
+ syntax: buffer overflow, make a smaller pattern please! (Max size = 2048)",
+							     name);
+						    return false;
+					    }
+				    }
+			    }
+			    else{
+				    if(*idx != ' '){
+					    snprintf(errbuf, errbuflen, 
+						     "Incorrect '%s' syntax \
+'%s': What is this \"%c\"(0x%X) doing in your binary buffer?  Valid hex \
+values only please (0x0 - 0xF) Position: %d", name, rule, (char) *idx, 
+						     (char) *idx, cnt);
+					    return false;
+				    }
+			    }
+		    }
+		    else{
+			    if(*idx >= 0x1F && *idx <= 0x7e){
+				    if(dummy_idx < dummy_end){
+					    tmp_buf[dummy_size] = 
+						    start_ptr[cnt];
+					    dummy_size++;
+				    }
+				    else{
+					    snprintf(errbuf, errbuflen, 
+						     "Incorrect '%s' syntax \
+'%s': dummy buffer overflow", name, rule);
+					    return false;
+				    }
+
+				    if(literal){
+					    literal = 0;
+				    }
+			    }
+			    else{
+				    if(literal){
+					    tmp_buf[dummy_size] = 
+						    start_ptr[cnt];
+					    dummy_size++;
+					    TRACE("Clearing literal");
+					    literal = 0;
+				    }
+				    else{
+					    snprintf(errbuf, errbuflen, 
+						     "Incorrect '%s' \
+syntax '%s': character value out of range, try a binary buffer",
+						     name, rule);
+					    return false;
+				    }
+			    }
+		    }
+		    break;
+	    }
+	    dummy_idx++;
+	    idx++;
+	    cnt++;
     }
     /* ...END BAD JUJU */
 
@@ -571,8 +653,8 @@ bool RulesParser::parseContent(char *rule, const char *name,
     }
 
     
-    if((ds_idx->pattern_buf = (unsigned char *) calloc(dummy_size+1, sizeof(char))) 
-       == NULL){
+    if((ds_idx->pattern_buf = 
+	(unsigned char *) calloc(dummy_size+1, sizeof(char))) == NULL){
 	    snprintf(errbuf, errbuflen,("Out of memory"));;
 	    return false;
     }
@@ -586,20 +668,8 @@ bool RulesParser::parseContent(char *rule, const char *name,
     return true;
 }
 
-void RulesParser::freeREs()
-{
-	freeRE(&ruleRE);
-	freeRE(&msgRE);
-	freeRE(&referenceRE);
-	freeRE(&fidRE);
-	freeRE(&revRE);
-	freeRE(&headerRE);
-	freeRE(&bodyRE);
-	freeRE(&urlRE);
-}
-
-
-void RulesParser::remove_substring(char *str, unsigned int b, unsigned int e)
+/*Remove substring starting in b and ending in e from a given string*/
+void RulesParser::removeSubstring(char *str, unsigned int b, unsigned int e)
 {
 	if (e == strlen(str))
 		str[b] = 0;
@@ -607,7 +677,7 @@ void RulesParser::remove_substring(char *str, unsigned int b, unsigned int e)
 		memmove(str + b, str + e, strlen(str) - e + 1);
 }
 
-
+/*Search for 'reference: ...' option, parse it and remove it from options.*/
 bool RulesParser::parseReference(char *options, Rule *rule)
 {
 	int rc;	
@@ -621,14 +691,18 @@ bool RulesParser::parseReference(char *options, Rule *rule)
 		return false;
 		
 	FK_ASSERT(rc == 3);
-		
-	Pcre_get_substring(options, ovector, rc, 1, (const char **)&ref.system);
-	Pcre_get_substring(options, ovector, rc, 2, (const char **)&ref.ruleId);
+
+	/*TODO: maybe it's a better idea to fail here more gently*/
+	Pcre_get_substring(options, ovector, rc, 1, 
+			   (const char **)&ref.system);
+	Pcre_get_substring(options, ovector, rc, 2, (const char **)&ref.refId);
 	rule->references.push_back(ref);	
-	remove_substring(options, ovector[0], ovector[1]);
+	removeSubstring(options, ovector[0], ovector[1]);
 	return true;
 }
 
+/*Search for option described by re, allocate and return buffer for it,
+  remove this option from options*/
 char *RulesParser::parseOption(pcre *re, char *options)
 {
 	int rc;	
@@ -642,10 +716,13 @@ char *RulesParser::parseOption(pcre *re, char *options)
 	FK_ASSERT(rc == 2);
 	
 	Pcre_get_substring(options, ovector, rc, 1, (const char **)&buf);
-	remove_substring(options, ovector[0], ovector[1]);
+	removeSubstring(options, ovector[0], ovector[1]);
 	return buf;
 }
 
+/*Search for '(url|body|headers)_content: ...' option (depending on 
+  first argument), parse it and remove it from options. 
+  Check if nocase flag is set for this content option*/
 char *RulesParser::parseContentOption(pcre *re, char *options, int *nocase)
 {
 	int rc;	
@@ -669,10 +746,11 @@ char *RulesParser::parseContentOption(pcre *re, char *options, int *nocase)
 	}
 	
 	Pcre_get_substring(options, ovector, rc, 1, (const char **)&buf);
-	remove_substring(options, ovector[0], ovector[1]);
+	removeSubstring(options, ovector[0], ovector[1]);
 	return buf;
 }
 
+/*Parse all rule's options*/
 char *RulesParser::parseOptions(char *options, Rule *rule)
 {	
 	char *res;
@@ -722,9 +800,11 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 		rule->bodyREs.push_back(re);		
 	}	
 	
-	while ((res = parseContentOption(urlContentRE, options, &cont.nocase))){
+	while ((res = parseContentOption(urlContentRE, options, 
+					 &cont.nocase))){
 		
-		if (!parseContent(res, "url_content:", &cont, errbuf, ERRBUFLEN)){
+		if (!parseContent(res, "url_content:", &cont, errbuf, 
+				  ERRBUFLEN)){
 			free(res);
 			return errbuf;
 		}
@@ -732,9 +812,11 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 		rule->url_content.push_back(cont);
 	}
 	
-	while ((res = parseContentOption(headersContentRE, options, &cont.nocase))){
+	while ((res = parseContentOption(headersContentRE, options, 
+					 &cont.nocase))){
 		
-		if (!parseContent(res, "headers_content:", &cont, errbuf, ERRBUFLEN)){
+		if (!parseContent(res, "headers_content:", &cont, errbuf, 
+				  ERRBUFLEN)){
 			free(res);
 			return errbuf;
 		}
@@ -742,9 +824,11 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 		rule->headers_content.push_back(cont);
 	}
 	
-	while ((res = parseContentOption(bodyContentRE, options, &cont.nocase))){
+	while ((res = parseContentOption(bodyContentRE, options, 
+					 &cont.nocase))){
 		
-		if (!parseContent(res, "body_content:", &cont, errbuf, ERRBUFLEN)){
+		if (!parseContent(res, "body_content:", &cont, errbuf, 
+				  ERRBUFLEN)){
 			free(res);
 			return errbuf;
 		}
@@ -754,16 +838,17 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 
 	while (parseReference(options, rule))
 		;
-	
+
+#ifdef DEBUG		
 	TRACE("MSG = %s", rule->msg);
 	TRACE("FID = %d", rule->fid);
 	TRACE("REV = %d", rule->rev);
 	TRACE("References");
 	list<RE>::iterator it;
 	list<reference>::iterator it2;
-	
-	for(it2 = rule->references.begin(); it2 != rule->references.end(); ++it2)
-		TRACE("\t %s : %s", it2->system, it2->ruleId);
+	for(it2 = rule->references.begin(); it2 != rule->references.end(); 
+	    ++it2)
+		TRACE("\t %s : %s", it2->system, it2->refId);
 	TRACE("Url rules");
 	for(it = rule->urlREs.begin(); it != rule->urlREs.end(); ++it)
 		TRACE("\t %s", it->pattern);
@@ -775,6 +860,9 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 		TRACE("\t %s", it->pattern);	
 	
 	TRACE("OPTIONS LEFT: %s", options);
+#endif /*DEBUG*/
+
+	/*some unrecognized options left*/
 	if (!emptyString(options)){
 		snprintf(errbuf, ERRBUFLEN, "Incorrect option syntax: %s", options);
 		return errbuf;
@@ -782,6 +870,13 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 	return NULL;
 }
 
+/*
+ * Parse line that is not commented nor empty so should contain correct rule.
+ * @rulebuf: input buffer
+ * @error: points to a static error message (don't free it) 
+ *          or NULL if rule is ok.
+ * return: Rule object on success, NULL on failure
+ */
 Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 {
 	Rule *rule;
@@ -801,29 +896,31 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 	errbuf[0] = 0;
 	
 		
-	rc = pcre_exec(ruleRE,         /* the compiled pattern */
-		       NULL,           /* no extra data - didn't study the pattern */
+	rc = pcre_exec(ruleRE,        /* the compiled pattern */
+		       NULL,     /* no extra data - didn't study the pattern */
 		       rulebuf,        /* the subject string */
 		       strlen(rulebuf),/* the length of the subject */
 		       0,              /* start at offset 0 in the subject */
 		       0,              /* default options */
-		       ovector,        /* output vector for substring information */
-		       OVECCOUNT);     /* number of elements in the output vector */
+		       ovector,   /* output vector for substring information */
+		       OVECCOUNT);/* number of elements in the output vector */
 	
 	if (rc < 0){
 		switch(rc){
 		case PCRE_ERROR_NOMATCH: 
-			snprintf(errbuf, ERRBUFLEN, "Rule has to have form: action (options)");
+			snprintf(errbuf, ERRBUFLEN, "Rule has to have form: "
+				 "action (options)");
 			break;
 		default: 
 			/*some other error*/
-			snprintf(errbuf, ERRBUFLEN, "Matching error %d\n", rc); 
+			snprintf(errbuf, ERRBUFLEN, "Matching error %d\n", 
+				 rc); 
 			break;
 		}
 		goto error;
 	}		
 	
-	/* Match succeded */		
+	/* Match succeeded */		
 	FK_ASSERT(rc == 3);
 			
 	TRACE("Match succeeded");
@@ -837,7 +934,8 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 		goto error;
 	}
 	
-	Pcre_get_substring(rulebuf, ovector, rc, 1, (const char **)&rule->action);
+	Pcre_get_substring(rulebuf, ovector, rc, 1, 
+			   (const char **)&rule->action);
 	
 	for(i = 0; i < NACTIONS; ++i)
 		if (strcmp(rule->action, knownActions[i]) == 0){
@@ -846,7 +944,8 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 		}
 	
 	if (i == NACTIONS){
-		snprintf(errbuf, ERRBUFLEN, "Unknown action %s ", rule->action);
+		snprintf(errbuf, ERRBUFLEN, "Unknown action %s ", 
+			 rule->action);
 		goto error;
 	}
 		
@@ -863,13 +962,18 @@ Rule* RulesParser::parseRule(const char *rulebuf, char **error)
 	return NULL;
 }
 
+/*
+  Parse single line
+  @rules: Rules object to which rule from this line is added (if this line contains 
+  correct rule)
+ */
 void RulesParser::parseLine(Rules *rules, char *line, int linenr)
 {
 	char *error;
 	Rule *newRule;
 	
 		
-	if (comentedString(line))
+	if (commentedOrEmptyString(line))
 		return;
 	
 	newRule = parseRule(line, &error);
@@ -886,14 +990,13 @@ void RulesParser::parseLine(Rules *rules, char *line, int linenr)
 
 /*
   Read rules from a given string.
-  @rules - Rules object to which succesfully read rules are added
-  @input - file pointer to read rules from
-  @error - pointer to an error message buffer or NULL if there is no error.
+  @rules: Rules object to which successfully read rules are added
+  @input: buffer with rules
+  @error: pointer to an error message buffer or NULL if there is no error.
            Error message is a static buffer which shouldn't
            be freed. If some rules have incorrect syntax, this 
-x	   routine creates suitable error message, but rules
+	   routine creates suitable error message, but rules
 	   that are correct are returned anyway.
-  @return - 
  */
 void RulesParser::parseRules(Rules *rules, const char *input, char **error)
 {
@@ -901,6 +1004,7 @@ void RulesParser::parseRules(Rules *rules, const char *input, char **error)
 	char buf[RULEBUFLEN];
 	int idx = 0;
 	int done = 0;
+	
 	errmsg[0] = 0;
 	*error = errmsg;	
 	
@@ -910,20 +1014,24 @@ void RulesParser::parseRules(Rules *rules, const char *input, char **error)
 	}
 	
 	if (!ruleRE){
-		err_append("Fatal error: ruleRE object wasn't successfully created");
+		err_append("Fatal error: ruleRE object wasn't "
+			   "successfully created");
 		return;
 	}
 
 	while(!done){	
 		linenr++;	
-		for(;input[idx] && input[idx] != '\n' && idx < RULEBUFLEN; idx++)
+		for(;input[idx] && input[idx] != '\n' && idx < RULEBUFLEN; 
+		    idx++)
 			buf[idx] = input[idx];
-		if (!input[idx])
-			done = 1;
+		
 		if (idx == RULEBUFLEN){
 			err_append("Line %d too long", linenr);
 			break;
 		}
+
+		if (!input[idx])
+			done = 1;
 		buf[idx] = 0;
 		parseLine(rules, buf, linenr);
 		input += idx + 1;
